@@ -25,7 +25,11 @@
 #include <config.h>
 #endif
 
+#include <glib.h>
+#include <gio/gio.h>
+
 #include "exchange-mapi-utils.h"
+#include "exchange-mapi-mail-utils.h"
 
 #ifdef G_OS_WIN32
 /* Undef the similar macro from pthread.h, it doesn't check if
@@ -172,7 +176,7 @@ exchange_mapi_util_find_SPropVal_array_namedid (struct SPropValue *values, Excha
 	g_return_val_if_fail (values != NULL, NULL);
 	g_return_val_if_fail (conn != NULL, NULL);
 
-	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid);
+	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid, NULL);
 	if (proptag != MAPI_E_RESERVED)
 		res = exchange_mapi_util_find_SPropVal_array_propval (values, proptag);
 
@@ -231,7 +235,7 @@ exchange_mapi_util_find_row_namedid (struct SRow *aRow, ExchangeMapiConnection *
 	g_return_val_if_fail (aRow != NULL, NULL);
 	g_return_val_if_fail (conn != NULL, NULL);
 
-	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid);
+	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid, NULL);
 	if (proptag != MAPI_E_RESERVED)
 		res = exchange_mapi_util_find_row_propval (aRow, proptag);
 
@@ -290,7 +294,7 @@ exchange_mapi_util_find_array_namedid (struct mapi_SPropValue_array *properties,
 	g_return_val_if_fail (properties != NULL, NULL);
 	g_return_val_if_fail (conn != NULL, NULL);
 
-	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid);
+	proptag = exchange_mapi_connection_resolve_named_prop (conn, fid, namedid, NULL);
 	if (proptag != MAPI_E_RESERVED)
 		res = exchange_mapi_util_find_array_propval (properties, proptag);
 
@@ -324,11 +328,16 @@ exchange_mapi_util_free_attachment_list (GSList **attach_list)
 
 	for (; l != NULL; l = l->next) {
 		ExchangeMAPIAttachment *attachment = (ExchangeMAPIAttachment *) (l->data);
-		/* FIXME: more stuff here */
-		g_free (attachment->lpProps);
-		exchange_mapi_util_free_stream_list (&(attachment->streams));
+
+		if (attachment->mail) {
+			mail_item_free (attachment->mail);
+		} else {
+			g_free (attachment->lpProps);
+			exchange_mapi_util_free_stream_list (&(attachment->streams));
+		}
+
 		g_free (attachment);
-		attachment = NULL;
+		l->data = NULL;
 	}
 	g_slist_free (*attach_list);
 	*attach_list = NULL;
@@ -654,14 +663,14 @@ bin_decode_string (const uint8_t *ptr, uint32_t sz, gchar **str, gboolean is_uni
 			break;
 	}
 
-	if (len >= sz || ptr[len] != 0x00 || (is_unicode && len + 1 >= sz && ptr[len + 1] != 0x00))
+	if (len >= sz || ptr[len] != 0x00 || (is_unicode && (len + 1 >= sz || ptr[len + 1] != 0x00)))
 		return 0;
 
 	if (is_unicode) {
 		*str = g_utf16_to_utf8 ((const gunichar2 *) ptr, len / 2, NULL, NULL, NULL);
 	} else {
 		*str = g_malloc0 (sizeof(gchar) * (1 + len));
-		strncpy (*str, (const gchar *) ptr, len - 1);
+		strncpy (*str, (const gchar *) ptr, len);
 	}
 
 	return len + 1 + (is_unicode ? 1 : 0);
@@ -718,7 +727,7 @@ static const uint8_t MAPI_ONE_OFF_UID[] = {
 #define MAPI_ONE_OFF_MYSTERY_FLAG 0x1000
 
 /**
- * e2k_entryid_generate_oneoff:
+ * exchange_mapi_util_recip_entryid_generate_smtp:
  * @entryid: entry ID to be filled
  * @display_name: the display name of the user
  * @email: the email address
@@ -730,7 +739,7 @@ static const uint8_t MAPI_ONE_OFF_UID[] = {
  * Return value: the recipient ENTRYID
  **/
 void
-exchange_mapi_util_entryid_generate_oneoff (TALLOC_CTX *mem_ctx, struct Binary_r *entryid, const gchar *display_name, const gchar *email)
+exchange_mapi_util_recip_entryid_generate_smtp (TALLOC_CTX *mem_ctx, struct Binary_r *entryid, const gchar *display_name, const gchar *email)
 {
 	g_return_if_fail (entryid != NULL);
 
@@ -743,8 +752,31 @@ exchange_mapi_util_entryid_generate_oneoff (TALLOC_CTX *mem_ctx, struct Binary_r
 	exchange_mapi_util_bin_append_unicode (mem_ctx, entryid, email);
 }
 
-gboolean
-exchange_mapi_util_entryid_decode_oneoff (const struct Binary_r *entryid, gchar **display_name, gchar **email)
+static const uint8_t MAPI_LOCAL_UID[] = {
+	0xdc, 0xa7, 0x40, 0xc8, 0xc0, 0x42, 0x10, 0x1a,
+	0xb4, 0xb9, 0x08, 0x00, 0x2b, 0x2f, 0xe1, 0x82
+};
+
+/**
+ * exchange_mapi_util_recip_entryid_generate_ex:
+ * @exchange_dn: the Exchange 5.5-style DN of the local user
+ *
+ * Constructs an ENTRYID value that can be used as a MAPI
+ * recipient (eg, for a message forwarding server-side rule),
+ * corresponding to the local user identified by @exchange_dn.
+ **/
+void
+exchange_mapi_util_recip_entryid_generate_ex (TALLOC_CTX *mem_ctx, struct Binary_r *entryid, const gchar *exchange_dn)
+{
+	exchange_mapi_util_bin_append_uint32 (mem_ctx, entryid, 0);
+	exchange_mapi_util_bin_append_val (mem_ctx, entryid, MAPI_LOCAL_UID, sizeof(MAPI_LOCAL_UID));
+	exchange_mapi_util_bin_append_uint16 (mem_ctx, entryid, 1);
+	exchange_mapi_util_bin_append_uint16 (mem_ctx, entryid, 0);
+	exchange_mapi_util_bin_append_string (mem_ctx, entryid, exchange_dn);
+}
+
+static gboolean
+recip_entryid_decode_smtp (const struct Binary_r *entryid, gchar **display_name, gchar **email)
 {
 	uint32_t u32, sz, r;
 	uint16_t u16, flags;
@@ -825,35 +857,102 @@ exchange_mapi_util_entryid_decode_oneoff (const struct Binary_r *entryid, gchar
 	return TRUE;
 }
 
-static const uint8_t MAPI_LOCAL_UID[] = {
-	0xdc, 0xa7, 0x40, 0xc8, 0xc0, 0x42, 0x10, 0x1a,
-	0xb4, 0xb9, 0x08, 0x00, 0x2b, 0x2f, 0xe1, 0x82
-};
+static gboolean
+recip_entryid_decode_ex (const struct Binary_r *entryid, gchar **exchange_dn)
+{
+	uint32_t u32, sz, r;
+	uint8_t *ptr;
+
+	g_return_val_if_fail (entryid != NULL, FALSE);
+	g_return_val_if_fail (entryid->lpb != NULL, FALSE);
+	g_return_val_if_fail (exchange_dn != NULL, FALSE);
+
+	*exchange_dn = NULL;
+
+	ptr = entryid->lpb;
+	sz = entryid->cb;
+
+	u32 = 1;
+	r = bin_decode_uint32 (ptr, sz, &u32);
+	if (!r || u32 != 0)
+		return FALSE;
+
+	ptr += r;
+	sz -= r;
+
+	for (r = 0; r < G_N_ELEMENTS (MAPI_LOCAL_UID) && r < sz; r++) {
+		if (ptr[r] != MAPI_LOCAL_UID[r])
+			return FALSE;
+	}
+
+	if (r != G_N_ELEMENTS (MAPI_LOCAL_UID))
+		return FALSE;
+
+	ptr += r;
+	sz -= r;
+
+	/* version */
+	u32 = 0;
+	r = bin_decode_uint32 (ptr, sz, &u32);
+	if (!r)
+		return FALSE;
+	ptr += r;
+	sz -= r;
+
+	/* type */
+	u32 = 0;
+	r = bin_decode_uint32 (ptr, sz, &u32);
+	if (!r)
+		return FALSE;
+	ptr += r;
+	sz -= r;
+
+	r = bin_decode_string (ptr, sz, exchange_dn, FALSE);
+	if (!r || !*exchange_dn)
+		return FALSE;
+
+	return TRUE;
+}
 
 /**
- * e2k_entryid_generate_local:
- * @exchange_dn: the Exchange 5.5-style DN of the local user
+ * exchange_mapi_util_recip_entryid_decode:
+ * @conn: ExchangeMapiCOnnection to resolve names, if required
+ * @entryid: recipient's ENTRYID to decode
+ * @display_name: (out): stored display name, if any; can be NULL
+ * @email: (out): email or exchange DN; cannot be NULL
  *
- * Constructs an ENTRYID value that can be used as a MAPI
- * recipient (eg, for a message forwarding server-side rule),
- * corresponding to the local user identified by @exchange_dn.
- *
- * Return value: the recipient ENTRYID
+ * Returns: Whether was able to decode recipient information from the @entryid.
  **/
-struct Binary_r *
-exchange_mapi_util_entryid_generate_local (TALLOC_CTX *mem_ctx, const gchar *exchange_dn)
+gboolean
+exchange_mapi_util_recip_entryid_decode (ExchangeMapiConnection *conn, const struct Binary_r *entryid, gchar **display_name, gchar **email)
 {
-	struct Binary_r *entryid;
+	gchar *dispnm = NULL, *exchange_dn = NULL;
 
-	entryid = talloc_zero (mem_ctx, struct Binary_r);
+	g_return_val_if_fail (conn != NULL, FALSE);
+	g_return_val_if_fail (entryid != NULL, FALSE);
+	g_return_val_if_fail (email != NULL, FALSE);
 
-	exchange_mapi_util_bin_append_uint32 (mem_ctx, entryid, 0);
-	exchange_mapi_util_bin_append_val (mem_ctx, entryid, MAPI_LOCAL_UID, sizeof(MAPI_LOCAL_UID));
-	exchange_mapi_util_bin_append_uint16 (mem_ctx, entryid, 1);
-	exchange_mapi_util_bin_append_uint16 (mem_ctx, entryid, 0);
-	exchange_mapi_util_bin_append_string (mem_ctx, entryid, exchange_dn);
+	*email = NULL;
+	if (display_name)
+		*display_name = NULL;
 
-	return entryid;
+	if (recip_entryid_decode_smtp (entryid, &dispnm, email)) {
+		if (display_name)
+			*display_name = dispnm;
+		else
+			g_free (dispnm);
+
+		return TRUE;
+	}
+
+	if (recip_entryid_decode_ex (entryid, &exchange_dn)) {
+		*email = exchange_mapi_connection_ex_to_smtp (conn, exchange_dn, display_name, NULL);
+		g_free (exchange_dn);
+
+		return *email != NULL;
+	}
+
+	return FALSE;
 }
 
 /**
@@ -900,7 +999,6 @@ exchange_lf_to_crlf (const gchar *in)
 gchar *
 exchange_crlf_to_lf (const gchar *in)
 {
-	gint len;
 	const gchar *s;
 	gchar *out;
 	GString *str;
@@ -909,7 +1007,6 @@ exchange_crlf_to_lf (const gchar *in)
 
 	str = g_string_new ("");
 
-	len = strlen (in);
 	for (s = in; *s; s++) {
 		if (*s != '\r')
 			str = g_string_append_c (str, *s);
@@ -930,7 +1027,7 @@ exchange_crlf_to_lf (const gchar *in)
  *
  * Constructs profile name from given parameters and
  * returns it as a newly allocated string. It can also
- * rename old profile name string to a new name, if requested. 
+ * rename old profile name string to a new name, if requested.
  **/
 gchar *
 exchange_mapi_util_profile_name (const gchar *username, const gchar *domain, const gchar *hostname, gboolean migrate)
@@ -947,7 +1044,7 @@ exchange_mapi_util_profile_name (const gchar *username, const gchar *domain, con
 		old_name = g_strdup_printf ("%s@%s", username, domain);
 		old_name = g_strcanon (old_name, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789@", '_');
 
-		RenameProfile (old_name, res);
+		exchange_mapi_rename_profile (old_name, res);
 
 		g_free (old_name);
 	}
@@ -988,7 +1085,7 @@ exchange_mapi_utils_add_named_ids_to_props_array (ExchangeMapiConnection *conn,
 	g_return_val_if_fail (named_ids_list != NULL, FALSE);
 	g_return_val_if_fail (named_ids_n_elems > 0, FALSE);
 
-	if (!exchange_mapi_connection_resolve_named_props (conn, fid, named_ids_list, named_ids_n_elems))
+	if (!exchange_mapi_connection_resolve_named_props (conn, fid, named_ids_list, named_ids_n_elems, NULL))
 		return FALSE;
 
 	for (i = 0; i < named_ids_n_elems; i++) {
@@ -1011,14 +1108,14 @@ exchange_mapi_utils_add_spropvalue (TALLOC_CTX *mem_ctx, struct SPropValue **val
 	g_return_val_if_fail (n_values != NULL, FALSE);
 
 	if ((prop_tag & 0xFFFF) == PT_DOUBLE) {
-		uint32_t zero = 0;
+		uint64_t zero = 0;
 
 		/* add an empty fake value and rewrite it */
 		*values_array = add_SPropValue (mem_ctx, *values_array, n_values, PROP_TAG(PT_LONG, 0x0001), &zero);
 
 		((*values_array)[(*n_values) - 1]).ulPropTag = prop_tag;
 		((*values_array)[(*n_values) - 1]).dwAlignPad = 0;
-		((*values_array)[(*n_values) - 1]).value.dbl = *((gdouble *) prop_value);
+		memcpy (&((*values_array)[(*n_values) - 1]).value.dbl, prop_value, 8);
 	} else {
 		*values_array = add_SPropValue (mem_ctx, *values_array, n_values, prop_tag, prop_value);
 	}
@@ -1038,7 +1135,7 @@ exchange_mapi_utils_add_spropvalue_named_id (ExchangeMapiConnection *conn, mapi_
 	g_return_val_if_fail (values_array != NULL, FALSE);
 	g_return_val_if_fail (n_values != NULL, FALSE);
 
-	prop_tag = exchange_mapi_connection_resolve_named_prop (conn, fid, named_id);
+	prop_tag = exchange_mapi_connection_resolve_named_prop (conn, fid, named_id, NULL);
 	if (prop_tag == MAPI_E_RESERVED)
 		return FALSE;
 
@@ -1108,3 +1205,65 @@ exchange_mapi_utils_push_crc32 (uint32_t crc32, uint8_t *bytes, uint32_t n_bytes
 
 	return crc32;
 }
+
+/* copies a Binary_r, which should be freed with exchange_mapi_util_free_binary_r() */
+struct Binary_r *
+exchange_mapi_util_copy_binary_r (const struct Binary_r *bin)
+{
+	struct Binary_r *res;
+
+	if (!bin || !bin->cb)
+		return NULL;
+
+	res = g_new0 (struct Binary_r, 1);
+	res->cb = bin->cb;
+	res->lpb = g_new (uint8_t, res->cb);
+	memcpy (res->lpb, bin->lpb, res->cb);
+
+	return res;
+}
+
+/* frees Binary_r previously allocated by exchange_mapi_util_copy_binary_r() */
+void
+exchange_mapi_util_free_binary_r (struct Binary_r *bin)
+{
+	if (!bin)
+		return;
+
+	g_free (bin->lpb);
+	g_free (bin);
+}
+
+time_t
+exchange_mapi_util_filetime_to_time_t (const struct FILETIME *filetime)
+{
+	NTTIME nt;
+
+	if (!filetime)
+		return (time_t) -1;
+
+	nt = filetime->dwHighDateTime;
+	nt = nt << 32;
+	nt |= filetime->dwLowDateTime;
+
+	nt /=  10 * 1000 * 1000;
+	nt -= 11644473600LL;
+
+	return (time_t) nt;
+}
+
+void
+exchange_mapi_util_time_t_to_filetime (const time_t tt, struct FILETIME *filetime)
+{
+	NTTIME nt;
+
+	g_return_if_fail (filetime != NULL);
+
+	nt = tt;
+	nt += 11644473600LL;
+	nt *=  10 * 1000 * 1000;
+
+	filetime->dwLowDateTime = nt & 0xFFFFFFFF;
+	nt = nt >> 32;
+	filetime->dwHighDateTime = nt & 0xFFFFFFFF;
+}
